<?php
// $Id: transformations_ui.pipeline.edit.inc,v 1.4 2009/03/30 19:50:01 jpetso Exp $
/**
 * @file
 * Transformations UI -
 * An interface for managing transformation pipelines.
 *
 * Copyright 2009 by Jakob Petsovits ("jpetso", http://drupal.org/user/56020)
 */


/**
 * Form callback for 'admin/build/transformations/%transformations_ui_persisted_pipeline':
 * A form to inspect & edit a pipeline.
 */
function transformations_ui_pipeline_edit(&$form_state, TfPipeline $pipeline) {
  // In order to overcome the limitations of the menu system (especially with
  // respect to local tasks which don't work well for child paths), this form
  // also acts as a hub for other forms to appear on the "Edit" tab.
  if (func_num_args() > 2) {
    $args = func_get_args();
    $page = $args[2];
    unset($args[2]);

    if ($page == 'input' && func_num_args() >= 5) {
      return call_user_func_array(
        'transformations_ui_pipeline_operation_input_edit', $args
      );
    }
  }

  drupal_add_css(drupal_get_path('module', 'transformations_ui') . '/transformations_ui.css');

  $pipelinePath = transformations_ui_pipeline_path($pipeline);

  $form = array();
  $form['#pipeline_id'] = transformations_ui_pipeline_persistence_id($pipeline);

  $blocks = array();

  $isEditingOperation = isset($_GET['edit-op']);
  $isEditingOperationInput = $isEditingOperation && $_GET['action'] == 'input';
  $isEditingOperationOutput = $isEditingOperation && $_GET['action'] == 'output';
  $isRenaming = $isEditingOperation && $_GET['action'] == 'rename';

  $isConnectingTo = array(
    'input' => ($isEditingOperation && $_GET['action'] == 'input'),
    'output' => ($isEditingOperation && $_GET['action'] == 'output'),
  );
  if ($isConnectingTo['input'] || $isConnectingTo['output']) {
    $connectingKey = $_GET['key'];
  }

  $operations = $pipeline->createOperations();

  // Create blocks for the regular operations, and also for
  // pipeline parameters/outputs.
  $blocks = array();
  $blocks[TfPipeline::Parameter] = array(
    'label' => t('Pipeline parameters'),
    'input' => array('keys' => array(), 'sources' => array()),
    'output' => array(
      'keys' => array(),
      'targets' => $pipeline->targets(TfPipeline::Parameter),
    ),
    'specialEntity' => TRUE,
  );
  foreach ($blocks[TfPipeline::Parameter]['output']['targets'] as $sourceKey => $targets) {
    $blocks[TfPipeline::Parameter]['output']['keys'][] = $sourceKey;
  }
  $blocks[TfPipeline::Parameter]['output']['keys'][] = TfPipeline::NewOutboundConnection;

  foreach ($operations as $operationId => $operation) {
    $blocks[$operationId] = array(
      'label' => $pipeline->operationProperty($operationId, 'label'),
      'input' => array(
        'keys' => $operation->inputKeys(),
        'sources' => $pipeline->sources($operationId),
      ),
      'output' => array(
        'keys' => $operation->outputKeys(),
        'targets' => $pipeline->targets($operationId),
      ),
    );
  }

  $blocks[TfPipeline::Output] = array(
    'label' => t('Pipeline outputs'),
    'input' => array(
      'keys' => array(),
      'sources' => $pipeline->sources(TfPipeline::Output),
    ),
    'output' => array('keys' => array(), 'targets' => array()),
    'specialEntity' => TRUE,
  );
  foreach ($blocks[TfPipeline::Output]['input']['sources'] as $targetKey => $source) {
    $blocks[TfPipeline::Output]['input']['keys'][] = $targetKey;
  }
  $blocks[TfPipeline::Output]['input']['keys'][] = TfPipeline::NewOutboundConnection;

  foreach ($blocks as $operationId => $block) {
    $operationElements[$operationId] = array(
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block">'
        . '<div class="transformations-operation-block-table">',
      '#suffix' => '</div></div>',
    );
    $operationElements[$operationId]['header'] = array(
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block-header">'
        . '<div class="transformations-operation-block-header-cell">',
      '#suffix' => '</div></div>',
    );
    $operationElements[$operationId]['header']['label'] = array(
      '#value' => check_plain($block['label']),
      '#prefix' => '<div class="transformations-operation-block-header-label">',
      '#suffix' => '</div>',
    );
    $operationElements[$operationId]['header']['actions'] = array(
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block-header-actions">',
      '#suffix' => '</div>',
    );

    // Operation editing functionality is only available for real operations.
    if ($isEditingOperation && $_GET['edit-op'] == $operationId) {
      $operationElements[$operationId]['header']['actions']['cancel'] = array(
        '#value' => transformations_ui_operation_action_image(
          'dialog-cancel', t('Cancel'), $pipelinePath
        ),
        '#weight' => 10,
      );
    }

    if (empty($block['specialEntity'])) {
      if ($isEditingOperation && $operationId == $_GET['edit-op']) {
        if ($isEditingOperationInput) {
          $operationElements[$operationId]['header']['actions']['update'] = array(
            '#value' => transformations_ui_operation_action_image(
              'document-properties', t('Assign fixed value'),
              $pipelinePath . '/edit/input/' . drupal_urlencode($operationId)
                . '/' . drupal_urlencode($_GET['key'])
            ),
          );
        }
        elseif ($isRenaming) {
          unset($operationElements[$operationId]['header']['label']['#value']);

          $operationElements[$operationId]['header']['label'] += array(
            '#type' => 'textfield',
            '#default_value' => $block['label'],
            '#size' => strlen($block['label']),
          );
          $operationElements[$operationId]['header']['actions']['update'] = array(
            '#type' => 'image_button',
            '#title' => t('Assign new name'),
            '#src' => drupal_get_path('module', 'transformations_ui') . '/images/dialog-ok.png',
            '#name' => 'remove-' . $operationId,
            '#tf_operation_id' => $operationId,
            '#submit' => array('transformations_ui_pipeline_operation_rename_submit'),
          );
        }
      }
      elseif (!isset($_GET['edit-op'])) {
        $operationElements[$operationId]['header']['actions']['rename'] = array(
          '#value' => transformations_ui_operation_action_image(
            'edit-rename', t('Rename operation'), $pipelinePath, array(
              'query' => 'edit-op=' . drupal_urlencode($operationId) . '&action=rename',
            )
          ),
        );
        $operationElements[$operationId]['header']['actions']['remove'] = array(
          '#value' => transformations_ui_operation_action_image(
            'edit-delete', t('Remove operation from pipeline'),
            $pipelinePath . '/remove-operation/' . $operationId
          ),
        );
      }
    }

    $i = 0;
    $max_actions_count = is_string($operationId) ? 2 : 1;
    while (count(element_children($operationElements[$operationId]['header']['actions'])) < $max_actions_count) {
      ++$i;
      // Have a non-empty value so that the form element is rendered at all.
      $operationElements[$operationId]['header']['actions']['pad' . $i] = array(
        '#value' => transformations_ui_operation_action_image('blank', ''),
        '#weight' => -100,
      );
    }

    $operationElements[$operationId]['connections'] = array(
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block-connections">'
        . '<div class="transformations-operation-block-connections-table">'
        . '<div class="transformations-operation-block-connections-body">',
      '#suffix' => '</div></div></div>',
    );
    $operationElements[$operationId]['connections']['input'] = array(
      '#type' => 'markup',
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block-inputs">',
      '#suffix' => '</div>',
    );
    $operationElements[$operationId]['connections']['output'] = array(
      '#type' => 'markup',
      '#value' => '',
      '#prefix' => '<div class="transformations-operation-block-outputs">',
      '#suffix' => '</div>',
    );

    $connectionElements = array();

    // Construct input/output connection elements, as form buttons.
    foreach (array('input', 'output') as $keyType) {
      $otherKeyType = ($keyType == 'input') ? 'output' : 'input';

      foreach ($block[$keyType]['keys'] as $key) {
        $classes = array();
        $styles = array();

        $isKeyConnecting = $isConnectingTo[$keyType] &&
          $operationId == $_GET['edit-op'] && $key == $connectingKey;

        $isConnectingCurrentPartner = FALSE;

        // Some key type specific differences.
        if ($keyType == 'input') { // a.k.a. "connection target"
          $source = $pipeline->source($operationId, $key);
          $isKeyConnected = !empty($source);

          if ($isKeyConnected && isset($source['key'])) {
            $sourceEntity = $source['entity'];
            $sourceKey = $source['key'];

            if ($isConnectingTo[$otherKeyType]
                && $_GET['edit-op'] == $sourceEntity && $connectingKey == $sourceKey) {
              $isConnectingCurrentPartner = TRUE;
            }
          }
        }
        elseif ($keyType == 'output') { // a.k.a. "connection source"
          $targets = $pipeline->targets($operationId, $key);
          $isKeyConnected = !empty($targets);

          $sourceEntity = $operationId;
          $sourceKey = $key;

          if ($isConnectingTo[$otherKeyType]) {
            foreach ($targets as $otherOperationId => $targetInfo) {
              foreach ($targetInfo as $otherKey => $true) {
                if ($_GET['edit-op'] == $otherOperationId && $connectingKey == $otherKey) {
                  $isConnectingCurrentPartner = TRUE;
                }
              }
            }
          }
        }

        $elementId = $keyType . '-' . $operationId . '-' . $key;
        unset($label);

        // Determine how to name the label. The thing that makes this a bit
        // more complicated is that we want to name the pipeline parameters and
        // pipeline outputs similar to the connected operation slots, and
        // we also need to special-case the virtual "New parameter/output" key.
        if ($operationId == TfPipeline::Parameter) {
          if ($key == TfPipeline::NewOutboundConnection) {
            $label = t('New parameter');
          }
          else {
            foreach ($targets as $targetEntity => $targetInfo) {
              if (isset($operations[$targetEntity])) {
                foreach ($targetInfo as $targetKey => $true) {
                  $label = $operations[$targetEntity]->inputProperty($targetKey, 'label');
                }
              }
            }
          }
        }
        elseif ($operationId == TfPipeline::Output) {
          if ($key == TfPipeline::NewOutboundConnection) {
            $label = t('New output');
          }
          elseif (isset($operations[$sourceEntity])) {
            $label = $operations[$sourceEntity]->outputProperty($sourceKey, 'label');
          }
        }
        elseif (isset($operations[$operationId])) {
          $label = ($keyType == 'input')
            ? $operations[$operationId]->inputProperty($key, 'label')
            : $operations[$operationId]->outputProperty($key, 'label');
        }
        if (!isset($label)) {
          $label = $key;
        }

        // The connector button for the input/output slot.
        $element = array(
          '#type' => 'submit',
          '#value' => $label,
          '#name' => $elementId,
          '#tf_operation_id' => $operationId,
          '#tf_key_type' => $keyType,
          '#tf_key' => $key,
          '#disabled' => $isEditingOperation &&
            (!$isConnectingTo[$otherKeyType] || $_GET['edit-op'] == $operationId),
          '#attributes' => array(),
          '#prefix' => '<div class="transformations-operation-' . $keyType . '">',
          '#suffix' => '</div>',
          '#submit' => array('transformations_ui_pipeline_operation_connect'),
        );

        // And now, a gazillion customizations depending on the current status
        // of the input/output slot and its connection.

        if ($operationId == TfPipeline::Parameter &&
            $isConnectingTo[$otherKeyType] && $_GET['edit-op'] == TfPipeline::Output) {
          $element['#disabled'] = TRUE;
        }
        elseif ($operationId == TfPipeline::Output &&
            $isConnectingTo[$otherKeyType] && $_GET['edit-op'] == TfPipeline::Parameter) {
          $element['#disabled'] = TRUE;
        }

        if ($isKeyConnecting) {
          $classes[] = 'connecting';

          if ($isKeyConnected) {
            $element['#disabled'] = FALSE;
            $element['#attributes']['title'] = t('Click to disconnect');
          }
        }

        if ($isConnectingTo[$otherKeyType] || $isKeyConnecting) {
          $element['#tf_connection'] = array(
            'entity' => $_GET['edit-op'],
            'key' => $connectingKey,
          );
          $classes[] = 'potential-connection-target';
        }

        if ($isKeyConnected) {
          $classes[] = 'connected';
        }

        unset($bgColor);
        if ($isKeyConnecting) {
          $bgColor = '#ffcc00';
        }
        elseif ($isConnectingCurrentPartner) {
          $bgColor = '#ffee99';
        }
        elseif ($isKeyConnected) {
          $bgColor = transformations_ui_connection_css_background_color(
            $sourceEntity, $sourceKey, $element['#disabled']
          );
        }
        if (isset($bgColor)) {
          $styles[] = "background-color: $bgColor";
        }

        $classes[] = $element['#disabled'] ? 'disabled' : 'enabled';

        $element['#attributes']['class'] = implode(' ', $classes);
        if (!empty($styles)) {
          $element['#attributes']['style'] = implode('; ', $styles);
        }
        $operationElements[$operationId]['connections'][$keyType][$elementId] = $element;
      } // end of input/output element construction
    } // end of foreach (array('input', 'output'))
  }

  if (empty($operationElements)) {
    $form['operations'] = array(
      '#value' => t('No operations have yet been defined for this pipeline.'),
    );
  }
  else {
    $form['operations'] = array(
      '#value' => '',
      '#prefix' => '<div class="clear-block transformations-operation-blocks">',
      '#suffix' => '</div>',
    );
    $form['operations'] += $operationElements;
  }

  if (!transformations_ui_is_pipeline_locked($pipeline) && $pipeline->isModified()) {
    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#submit' => array('transformations_ui_pipeline_edit_save'),
      '#disabled' => $isEditingOperation,
    );
    $form['discard'] = array(
      '#type' => 'submit',
      '#value' => t('Discard changes'),
      '#submit' => array('transformations_ui_pipeline_edit_discard'),
      '#disabled' => $isEditingOperation,
    );
  }
  else {
    $form['cancel'] = array(
      '#type' => 'submit',
      '#value' => t('Done'),
      '#submit' => array('transformations_ui_goto_submit'),
      '#goto_path' => 'admin/build/transformations',
      '#disabled' => $isEditingOperation,
    );
  }

  if (!transformations_ui_is_pipeline_locked($pipeline) && $pipeline->id()) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#submit' => array('transformations_ui_pipeline_edit_delete'),
      '#disabled' => $isEditingOperation,
    );
  }

  return $form;
}

/**
 * Submit callback for a pipeline's "Save" button.
 */
function transformations_ui_pipeline_edit_save($form, &$form_state) {
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);

  if (!$pipeline) {
    drupal_set_message(t('Error loading the pipeline.'), 'error');
    return;
  }
  transformations_ui_pipeline_save($pipeline);
  drupal_set_message(t('%pipeline has been saved.', array(
    '%pipeline' => $pipeline->name(),
  )));
  $form_state['redirect'] = transformations_ui_pipeline_path($pipeline);
}

/**
 * Submit callback for a pipeline's "Discard changes" button.
 */
function transformations_ui_pipeline_edit_discard($form, &$form_state) {
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);

  if (!$pipeline) {
    drupal_set_message(t('Error loading the pipeline.'), 'error');
    return;
  }
  transformations_ui_pipeline_discard($pipeline);

  if ($pipeline->id()) {
    drupal_set_message(t('%pipeline has restored to its saved state.', array(
      '%pipeline' => $pipeline->name(),
    )));
  }
  else {
    drupal_set_message(t('%pipeline has been discarded.', array(
      '%pipeline' => $pipeline->name(),
    )));
  }
}

/**
 * Submit callback for a pipeline's "Delete" button.
 */
function transformations_ui_pipeline_edit_delete($form, &$form_state) {
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);
  $form_state['redirect'] = transformations_ui_pipeline_path($pipeline) . '/delete';
}

/**
 * Return markup for an icon that is supposed to be inserted in the
 * 'actions' div of an operation block header.
 */
function transformations_ui_operation_action_image($icon, $title, $path = NULL, $l_options = array()) {
  $image = theme('image',
    drupal_get_path('module', 'transformations_ui') . "/images/$icon.png",
    $title, $title
  );
  if (!isset($path)) {
    return $image;
  }
  return l($image, $path, array('html' => TRUE) + $l_options);
}

/**
 * Submit handler for the "Rename" ("ok") button of an operation.
 */
function transformations_ui_pipeline_operation_rename_submit($form, &$form_state) {
  $operationId = $form_state['clicked_button']['#tf_operation_id'];
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);
  $pipeline->setOperationProperty($operationId, 'label', $form_state['values']['label']);
  transformations_ui_pipeline_persist($pipeline);
}

/**
 * Submit callback for an input or output connection button.
 */
function transformations_ui_pipeline_operation_connect($form, &$form_state) {
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);

  $operationId = $form_state['clicked_button']['#tf_operation_id'];
  $keyType = $form_state['clicked_button']['#tf_key_type'];
  $key = $form_state['clicked_button']['#tf_key'];

  $otherConnection = isset($form_state['clicked_button']['#tf_connection'])
    ? $form_state['clicked_button']['#tf_connection']
    : array();

  if (!empty($otherConnection)) {
    // $key and $keyType are from the second clicked button,
    // and $form_state['clicked_button']['#tf_connection'] is from the first.
    if ($keyType == 'input') {
      $source = $otherConnection;
      $target = array(
        'entity' => $operationId,
        'key' => $key,
      );
    }
    elseif ($keyType == 'output') {
      $source = array(
        'entity' => $operationId,
        'key' => $key,
      );
      $target = $otherConnection;
    }

    // New parameter and output names are entered into the textfield,
    // so let's use those instead of TfPipeline::NewOutboundConnection.
    if ($source['entity'] == TfPipeline::Parameter
        && $source['key'] == TfPipeline::NewOutboundConnection) {
      $source['key'] = $target['key'];
    }
    elseif ($target['entity'] == TfPipeline::Output
        && $target['key'] == TfPipeline::NewOutboundConnection) {
      $target['key'] = $source['key'];
    }

    if ($source['entity'] == $target['entity']) {
      if ($keyType == 'input') {
        $pipeline->disconnectTarget($operationId, $key);
      }
      elseif ($keyType == 'output') {
        $pipeline->disconnectSource($operationId, $key);
      }
    }
    else {
      $pipeline->connect(
        $source['entity'], $source['key'], $target['entity'], $target['key']
      );
    }

    transformations_ui_pipeline_persist($pipeline);
    $form_state['redirect'] = transformations_ui_pipeline_path($pipeline);
  }
  else {
    $form_state['redirect'] = array(
      transformations_ui_pipeline_path($pipeline),
      'edit-op=' . drupal_urlencode($operationId) . '&action=' . $keyType
      . '&key=' . drupal_urlencode($key)
    );
    unset($form_state['edit']); // For drupal_goto(). Don't ask.
  }
}

/**
 * Page callback for the "Remove" button on each operation.
 */
function transformations_ui_pipeline_operation_remove(TfPipeline $pipeline, $operationId) {
  $pipeline->removeOperation($operationId);
  transformations_ui_pipeline_persist($pipeline);
  drupal_goto(transformations_ui_pipeline_path($pipeline));
}

/**
 * Calculate a color for a given @p $entity and @p $key of a connection.
 * Preferably, you would always use the values of the connection source,
 * because each source can be assigned to multiple targets but a target can
 * only be assigned to a single source. (In other words, by using the source
 * entity and key, you get the same color for the same data.)
 *
 * @return
 *   An array with two elements, the foreground and background values. Assign
 *   them with @code list($bgColor, $fgColor) = (...function call...) @endcode.
 */
function transformations_ui_connection_css_background_color($entity, $key, $disabled = FALSE) {
  $r = 192 + (abs(crc32(md5($entity . '::' . $key))) % 48);
  $g = 192 + (abs(crc32(md5($key))) % 50);
  $b = 192 + (abs(crc32(md5($entity))) % 64);

  if (abs($g - $r) < 10) {
    $r -= 10;
    $g = min($g + 20, 255);
  }
  if (abs($b - $g) < 10) {
    $g -= 10;
    $b = min($b + 20, 255);
  }

  if ($disabled) {
    $r = min($r + 20, 255);
    $g = min($g + 20, 255);
    $b = min($b + 20, 255);
  }
  return "rgb($r, $g, $b)";
}


/**
 * Form callback for assigning fixed data to an operation input, at
 * 'admin/build/transformations/%transformations_ui_persisted_pipeline/edit/input/%/%':
 */
function transformations_ui_pipeline_operation_input_edit(&$form_state, TfPipeline $pipeline, $operationId, $inputKey) {
  $pipelineOperation = TfOperation::load('TfPipelineOperation', array(
    'pipeline' => $pipeline,
  ));
  if (!$pipelineOperation) {
    drupal_set_message(t('Pipeline operation could not be initialized.'), 'error');
    return array();
  }
  $operation = $pipelineOperation->operation($operationId);

  $form_element = TfDataWidget::forInput($operation, $inputKey);
  if (!$form_element) {
    drupal_set_message(t('No input with key %input-key exists for this operation.', array('%input-key' => $inputKey)), 'error');
    return array();
  }

  $form_element['#title'] = t(
    'Assign value to input "@input-name" (operation "@operation-name")', array(
      '@input-name' => $operation->inputProperty($inputKey, 'label'),
      '@operation-name' => $pipeline->operationProperty($operationId, 'label'),
  ));

  $form = array();
  $locked = transformations_ui_is_pipeline_locked($pipeline);

  // If there is an existing source connection, warn the user.
  $source = $pipeline->source($operationId, $inputKey);
  if ($source && isset($source['key']) && !$locked) {
    if ($source['entity'] == TfPipeline::Parameter) {
      $message = t('This input is already connected to a pipeline parameter. If you assign a new value, the input will be disconnected from that parameter. !click-here to go back without making changes.', array(
        '!click-here' => l(t('Click here'), transformations_ui_pipeline_path($pipeline)),
      ));
    }
    else {
      $sourceOperation = $pipelineOperation->operation($sourceOperationId);
      $message = t('This input is already connected to the @input-key output of the operation "@operation-name". If you assign a new value, the input will be disconnected from that operation output. !click-here to go back without making changes.', array(
        '@input-key' => $sourceOperation->inputProperty($source['key'], 'label'),
        '@operation-name' => $pipeline->operationProperty($sourceOperationId, 'label'),
        '!click-here' => l(t('Click here'), transformations_ui_pipeline_path($pipeline)),
      ));
    }
    $form['warning'] = array(
      '#value' => $message,
      '#prefix' => '<div class="messages warning">',
      '#suffix' => '</div>',
    );
  }

  $form['#pipeline_id'] = transformations_ui_pipeline_persistence_id($pipeline);
  $form['#operation_id'] = $operationId;
  $form['#input_key'] = $inputKey;

  $form['input'] = $form_element;

  $form['actions'] = array(
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  if (!$locked) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Store value'),
      '#submit' => array('transformations_ui_pipeline_operation_input_edit_submit'),
    );
  }
  $form['cancel'] = array(
    '#type' => 'markup',
    '#value' => l($locked ? t('Back') : t('Cancel'),
      transformations_ui_pipeline_path($pipeline)
    ),
  );
  return $form;
}

/**
 * Submit callback for the "Assign fixed input data" form.
 */
function transformations_ui_pipeline_operation_input_edit_submit(&$form, &$form_state) {
  $pipeline = transformations_ui_persisted_pipeline_load($form['#pipeline_id']);
  $operationId = $form['#operation_id'];
  $inputKey = $form['#input_key'];
  $widgetValue = $form_state['values']['input'];

  if ($widgetValue->useDefault()) {
    $pipeline->disconnectTarget($operationId, $inputKey);
  }
  else {
    $pipeline->connect(TfPipeline::Data, $widgetValue->value(), $operationId, $inputKey);
  }
  transformations_ui_pipeline_persist($pipeline);
  $form_state['redirect'] = transformations_ui_pipeline_path($pipeline);
}
